// Generated from player/lua/select.lua

"--[[\n"
"This file is part of mpv.\n"
"\n"
"mpv is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU Lesser General Public\n"
"License as published by the Free Software Foundation; either\n"
"version 2.1 of the License, or (at your option) any later version.\n"
"\n"
"mpv is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU Lesser General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU Lesser General Public\n"
"License along with mpv.  If not, see <http://www.gnu.org/licenses/>.\n"
"]]\n"
"\n"
"local utils = require \"mp.utils\"\n"
"local input = require \"mp.input\"\n"
"\n"
"local function show_error(message)\n"
"    mp.msg.error(message)\n"
"    if mp.get_property_native(\"vo-configured\") then\n"
"        mp.osd_message(message)\n"
"    end\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-playlist\", function ()\n"
"    local playlist = {}\n"
"    local default_item\n"
"    local show = mp.get_property_native(\"osd-playlist-entry\")\n"
"    local trailing_slash_pattern = mp.get_property(\"platform\") == \"windows\"\n"
"                                   and \"[/\\\\]+\044\" or \"/+\044\"\n"
"\n"
"    for i, entry in ipairs(mp.get_property_native(\"playlist\")) do\n"
"        playlist[i] = entry.title\n"
"        if not playlist[i] or show ~= \"title\" then\n"
"            playlist[i] = entry.filename\n"
"            if not playlist[i]:find(\"://\") then\n"
"                playlist[i] = select(2, utils.split_path(\n"
"                    playlist[i]:gsub(trailing_slash_pattern, \"\")))\n"
"            end\n"
"        end\n"
"        if entry.title and show == \"both\" then\n"
"            playlist[i] = string.format(\"%s (%s)\", entry.title, playlist[i])\n"
"        end\n"
"\n"
"        if entry.playing then\n"
"            default_item = i\n"
"        end\n"
"    end\n"
"\n"
"    if #playlist == 0 then\n"
"        show_error(\"The playlist is empty.\")\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a playlist entry:\",\n"
"        items = playlist,\n"
"        default_item = default_item,\n"
"        submit = function (index)\n"
"            mp.commandv(\"playlist-play-index\", index - 1)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local function format_flags(track)\n"
"    local flags = \"\"\n"
"\n"
"    for _, flag in ipairs({\n"
"        \"default\", \"forced\", \"dependent\", \"visual-impaired\", \"hearing-impaired\",\n"
"        \"image\", \"external\"\n"
"    }) do\n"
"        if track[flag] then\n"
"            flags = flags .. flag .. \" \"\n"
"        end\n"
"    end\n"
"\n"
"    if flags == \"\" then\n"
"        return \"\"\n"
"    end\n"
"\n"
"    return \" [\" .. flags:sub(1, -2) .. \"]\"\n"
"end\n"
"\n"
"local function format_track(track)\n"
"    local bitrate = track[\"demux-bitrate\"] or track[\"hls-bitrate\"]\n"
"\n"
"    return (track.selected and \"\342\227\217\" or \"\342\227\213\") ..\n"
"        (track.title and \" \" .. track.title or \"\") ..\n"
"        \" (\" .. (\n"
"            (track.lang and track.lang .. \" \" or \"\") ..\n"
"            (track.codec and track.codec .. \" \" or \"\") ..\n"
"            (track[\"demux-w\"] and track[\"demux-w\"] .. \"x\" .. track[\"demux-h\"]\n"
"             .. \" \" or \"\") ..\n"
"            (track[\"demux-fps\"] and not track.image\n"
"             and string.format(\"%.4f\", track[\"demux-fps\"]):gsub(\"%.\0770*\044\", \"\") ..\n"
"             \" fps \" or \"\") ..\n"
"            (track[\"demux-channel-count\"] and track[\"demux-channel-count\"] ..\n"
"             \"ch \" or \"\") ..\n"
"            (track[\"codec-profile\"] and track.type == \"audio\"\n"
"             and track[\"codec-profile\"] .. \" \" or \"\") ..\n"
"            (track[\"demux-samplerate\"] and track[\"demux-samplerate\"] / 1000 ..\n"
"             \" kHz \" or \"\") ..\n"
"            (bitrate and string.format(\"%.0f\", bitrate / 1000) ..\n"
"             \" kbps \" or \"\")\n"
"        ):sub(1, -2) .. \")\" .. format_flags(track)\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-track\", function ()\n"
"    local tracks = {}\n"
"\n"
"    for i, track in ipairs(mp.get_property_native(\"track-list\")) do\n"
"        tracks[i] = (track.image and \"Image\" or\n"
"                     track.type:sub(1, 1):upper() .. track.type:sub(2)) .. \": \" ..\n"
"                    format_track(track)\n"
"    end\n"
"\n"
"    if #tracks == 0 then\n"
"        show_error(\"No available tracks.\")\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a track:\",\n"
"        items = tracks,\n"
"        submit = function (id)\n"
"            local track = mp.get_property_native(\"track-list/\" .. id - 1)\n"
"            if track then\n"
"                mp.set_property(track.type, track.selected and \"no\" or track.id)\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local function select_track(property, type, prompt, error)\n"
"    local tracks = {}\n"
"    local items = {}\n"
"    local default_item\n"
"    local track_id = mp.get_property_native(property)\n"
"\n"
"    for _, track in ipairs(mp.get_property_native(\"track-list\")) do\n"
"        if track.type == type then\n"
"            tracks[#tracks + 1] = track\n"
"            items[#items + 1] = format_track(track)\n"
"\n"
"            if track.id == track_id then\n"
"                default_item = #items\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    if #items == 0 then\n"
"        show_error(error)\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = prompt,\n"
"        items = items,\n"
"        default_item = default_item,\n"
"        submit = function (id)\n"
"            mp.set_property(property, tracks[id].selected and \"no\" or tracks[id].id)\n"
"        end,\n"
"    })\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-sid\", function ()\n"
"    select_track(\"sid\", \"sub\", \"Select a subtitle:\", \"No available subtitles.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-secondary-sid\", function ()\n"
"    select_track(\"secondary-sid\", \"sub\", \"Select a secondary subtitle:\",\n"
"                 \"No available subtitles.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-aid\", function ()\n"
"    select_track(\"aid\", \"audio\", \"Select an audio track:\",\n"
"                 \"No available audio tracks.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-vid\", function ()\n"
"    select_track(\"vid\", \"video\", \"Select a video track:\",\n"
"                 \"No available video tracks.\")\n"
"end)\n"
"\n"
"local function format_time(t, duration)\n"
"    local h = math.floor(t / (60 * 60))\n"
"    t = t - (h * 60 * 60)\n"
"    local m = math.floor(t / 60)\n"
"    local s = t - (m * 60)\n"
"\n"
"    if duration >= 60 * 60 or h > 0 then\n"
"        return string.format(\"%.2d:%.2d:%.2d\", h, m, s)\n"
"    end\n"
"\n"
"    return string.format(\"%.2d:%.2d\", m, s)\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-chapter\", function ()\n"
"    local chapters = {}\n"
"    local default_item = mp.get_property_native(\"chapter\")\n"
"\n"
"    if default_item == nil then\n"
"        show_error(\"No available chapters.\")\n"
"        return\n"
"    end\n"
"\n"
"    local duration = mp.get_property_native(\"duration\", math.huge)\n"
"\n"
"    for i, chapter in ipairs(mp.get_property_native(\"chapter-list\")) do\n"
"        chapters[i] = format_time(chapter.time, duration) .. \" \" .. chapter.title\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a chapter:\",\n"
"        items = chapters,\n"
"        default_item = default_item + 1,\n"
"        submit = function (chapter)\n"
"            mp.set_property(\"chapter\", chapter - 1)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-subtitle-line\", function ()\n"
"    local sub = mp.get_property_native(\"current-tracks/sub\")\n"
"\n"
"    if sub == nil then\n"
"        show_error(\"No subtitle is loaded.\")\n"
"        return\n"
"    end\n"
"\n"
"    if sub.external and sub[\"external-filename\"]:find(\"^edl://\") then\n"
"        sub[\"external-filename\"] = sub[\"external-filename\"]:match('https\077://.*')\n"
"                                   or sub[\"external-filename\"]\n"
"    end\n"
"\n"
"    local r = mp.command_native({\n"
"        name = \"subprocess\",\n"
"        capture_stdout = true,\n"
"        args = sub.external\n"
"            and {\"ffmpeg\", \"-loglevel\", \"error\", \"-i\", sub[\"external-filename\"],\n"
"                 \"-f\", \"lrc\", \"-map_metadata\", \"-1\", \"-fflags\", \"+bitexact\", \"-\"}\n"
"            or {\"ffmpeg\", \"-loglevel\", \"error\", \"-i\", mp.get_property(\"path\"),\n"
"                \"-map\", \"s:\" .. sub[\"id\"] - 1, \"-f\", \"lrc\", \"-map_metadata\",\n"
"                \"-1\", \"-fflags\", \"+bitexact\", \"-\"}\n"
"    })\n"
"\n"
"    if r.error_string == \"init\" then\n"
"        show_error(\"Failed to extract subtitles: ffmpeg not found.\")\n"
"        return\n"
"    elseif r.status ~= 0 then\n"
"        show_error(\"Failed to extract subtitles.\")\n"
"        return\n"
"    end\n"
"\n"
"    local sub_lines = {}\n"
"    local sub_times = {}\n"
"    local default_item\n"
"    local delay = mp.get_property_native(\"sub-delay\")\n"
"    local time_pos = mp.get_property_native(\"time-pos\") - delay\n"
"    local duration = mp.get_property_native(\"duration\", math.huge)\n"
"\n"
"    -- Strip HTML and ASS tags.\n"
"    for line in r.stdout:gsub(\"<.->\", \"\"):gsub(\"{\\\\.-}\", \"\"):gmatch(\"[^\\n]+\") do\n"
"        -- ffmpeg outputs LRCs with minutes > 60 instead of adding hours.\n"
"        sub_times[#sub_times + 1] = line:match(\"%d+\") * 60 + line:match(\":([%d%.]*)\")\n"
"        sub_lines[#sub_lines + 1] = format_time(sub_times[#sub_times], duration) ..\n"
"                                    \" \" .. line:gsub(\".*]\", \"\", 1)\n"
"\n"
"        if sub_times[#sub_times] <= time_pos then\n"
"            default_item = #sub_times\n"
"        end\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a line to seek to:\",\n"
"        items = sub_lines,\n"
"        default_item = default_item,\n"
"        submit = function (index)\n"
"            -- Add an offset to seek to the correct line while paused without a\n"
"            -- video track.\n"
"            if mp.get_property_native(\"current-tracks/video/image\") ~= false then\n"
"                delay = delay + 0.1\n"
"            end\n"
"\n"
"            mp.commandv(\"seek\", sub_times[index] + delay, \"absolute\")\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-audio-device\", function ()\n"
"    local devices = mp.get_property_native(\"audio-device-list\")\n"
"    local items = {}\n"
"    -- This is only useful if an --audio-device has been explicitly set,\n"
"    -- otherwise its value is just auto and there is no current-audio-device\n"
"    -- property.\n"
"    local selected_device = mp.get_property(\"audio-device\")\n"
"    local default_item\n"
"\n"
"    if #devices == 0 then\n"
"        show_error(\"No available audio devices.\")\n"
"        return\n"
"    end\n"
"\n"
"    for i, device in ipairs(devices) do\n"
"        items[i] = device.name .. \" (\" .. device.description .. \")\"\n"
"\n"
"        if device.name == selected_device then\n"
"            default_item = i\n"
"        end\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select an audio device:\",\n"
"        items = items,\n"
"        default_item = default_item,\n"
"        submit = function (id)\n"
"            mp.set_property(\"audio-device\", devices[id].name)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-binding\", function ()\n"
"    local bindings = {}\n"
"\n"
"    for _, binding in pairs(mp.get_property_native(\"input-bindings\")) do\n"
"        if binding.priority >= 0 and (\n"
"               bindings[binding.key] == nil or\n"
"               (bindings[binding.key].is_weak and not binding.is_weak) or\n"
"               (binding.is_weak == bindings[binding.key].is_weak and\n"
"                binding.priority > bindings[binding.key].priority)\n"
"        ) then\n"
"            bindings[binding.key] = binding\n"
"        end\n"
"    end\n"
"\n"
"    local items = {}\n"
"    for _, binding in pairs(bindings) do\n"
"        if binding.cmd ~= \"ignore\" then\n"
"            items[#items + 1] = binding.key .. \" \" .. binding.cmd\n"
"        end\n"
"    end\n"
"\n"
"    table.sort(items)\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a binding:\",\n"
"        items = items,\n"
"        submit = function (i)\n"
"            mp.command(items[i]:gsub(\"^.- \", \"\"))\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local properties = {}\n"
"\n"
"local function add_property(property, value)\n"
"    value = value or mp.get_property_native(property)\n"
"\n"
"    if type(value) == \"table\" and next(value) then\n"
"        for key, val in pairs(value) do\n"
"            add_property(property .. \"/\" .. key, val)\n"
"        end\n"
"    else\n"
"        properties[#properties + 1] = property .. \": \" .. utils.to_string(value)\n"
"    end\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"show-properties\", function ()\n"
"    properties = {}\n"
"\n"
"    -- Don't log errors for renamed and removed properties.\n"
"    local msg_level_backup = mp.get_property(\"msg-level\")\n"
"    mp.set_property(\"msg-level\", msg_level_backup == \"\" and \"cplayer=no\"\n"
"                                 or msg_level_backup .. \",cplayer=no\")\n"
"\n"
"    for _, property in pairs(mp.get_property_native(\"property-list\")) do\n"
"        add_property(property)\n"
"    end\n"
"\n"
"    mp.set_property(\"msg-level\", msg_level_backup)\n"
"\n"
"    add_property(\"current-tracks/audio\")\n"
"    add_property(\"current-tracks/video\")\n"
"    add_property(\"current-tracks/sub\")\n"
"    add_property(\"current-tracks/sub2\")\n"
"\n"
"    table.sort(properties)\n"
"\n"
"    input.select({\n"
"        prompt = \"Inspect a property:\",\n"
"        items = properties,\n"
"        submit = function (i)\n"
"            if mp.get_property_native(\"vo-configured\") then\n"
"                mp.commandv(\"expand-properties\", \"show-text\",\n"
"                            (#properties[i] > 100 and\n"
"                             \"\044{osd-ass-cc/0}{\\\\fs9}\044{osd-ass-cc/1}\" or \"\") ..\n"
"                            \"\044>\" .. properties[i], 20000)\n"
"            else\n"
"                mp.msg.info(properties[i])\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
